Imports System.Runtime.InteropServices


Module ModuloUSB

    Public MyHID As New Hid()
    'Used in error messages.
    Const ModuleName As String = "FuncionesUSB"

    Public Servo(29) As Byte        'Esta variable contiene las posiciones 

    Public SendData(63) As Byte
    Public MyDeviceDetected As Boolean
    Public HIDHandle As Integer
    Public ReadHandle As Integer
    Public WriteHandle As Integer
    Public MyDeviceManagement As New DeviceManagement()

    Public DeviceNotificationHandle As IntPtr
    Public ExclusiveAccess As Boolean
    Public HIDUsage As String
    Public MyDevicePathName As String


    Public Function AbrirUSB(ByVal MostraMensaje As Boolean) As Boolean

        'Purpose    : Uses a series of API calls to locate a HID-class device
        '           ; by its Vendor ID and Product ID.

        'Returns    : True if the device is detected, False if not detected.

        'Dim Count As Short
        Dim DeviceFound As Boolean
        Dim DevicePathName(127) As String
        Dim GUIDString As String
        Dim HidGuid As System.Guid
        Dim LastDevice As Boolean
        Dim MemberIndex As Integer
        Dim MyProductID As Short
        Dim MyVendorID As Short
        Dim Result As Boolean
        Dim Security As SECURITY_ATTRIBUTES
        Dim Success As Boolean

        Try

            HidGuid = Guid.Empty
            LastDevice = False
            MyDeviceDetected = False

            'Values for the SECURITY_ATTRIBUTES structure:
            Security.lpSecurityDescriptor = 0
            Security.bInheritHandle = CInt(True)
            Security.nLength = Len(Security)

            'Get the device's Vendor ID and Product ID from the form's text boxes.
            MyVendorID = &H1562
            MyProductID = &H2312

            '***
            'API function: 'HidD_GetHidGuid

            'Purpose: Retrieves the interface class GUID for the HID class.

            'Accepts: 'A System.Guid object for storing the GUID.
            '***

            HidD_GetHidGuid(HidGuid)

            'Display the GUID.
            GUIDString = HidGuid.ToString

            'Fill an array with the device path names of all attached HIDs.
            DeviceFound = MyDeviceManagement.FindDeviceFromGuid _
                (HidGuid, _
                DevicePathName)

            'If there is at least one HID, attempt to read the Vendor ID and Product ID
            'of each device until there is a match or all devices have been examined.

            If DeviceFound = True Then
                MemberIndex = 0
                Do
                    '***
                    'API function:
                    'CreateFile

                    'Purpose:
                    'Retrieves a handle to a device.

                    'Accepts:
                    'A device path name returned by SetupDiGetDeviceInterfaceDetail
                    'The type of access requested (read/write).
                    'FILE_SHARE attributes to allow other processes to access the device while this handle is open.
                    'A Security structure. Using Null for this may cause problems under Windows XP.
                    'A creation disposition value. Use OPEN_EXISTING for devices.
                    'Flags and attributes for files. Not used for devices.
                    'Handle to a template file. Not used.

                    'Returns: a handle without read or write access.
                    'This enables obtaining information about all HIDs, even system
                    'keyboards and mice. 
                    'Separate handles are used for reading and writing.
                    '***

                    HIDHandle = CreateFile _
                        (DevicePathName(MemberIndex), _
                        0, _
                        FILE_SHARE_READ Or FILE_SHARE_WRITE, _
                        Security, _
                        OPEN_EXISTING, _
                        0, _
                        0)

                    If (HIDHandle <> INVALID_HANDLE_VALUE) Then

                        'The returned handle is valid, 
                        'so find out if this is the device we're looking for.

                        'Set the Size property of DeviceAttributes to the number of bytes in the structure.
                        MyHID.DeviceAttributes.Size = Marshal.SizeOf(MyHID.DeviceAttributes)

                        '***
                        'API function:
                        'HidD_GetAttributes

                        'Purpose:
                        'Retrieves a HIDD_ATTRIBUTES structure containing the Vendor ID, 
                        'Product ID, and Product Version Number for a device.

                        'Accepts:
                        'A handle returned by CreateFile.
                        'A pointer to receive a HIDD_ATTRIBUTES structure.

                        'Returns:
                        'True on success, False on failure.
                        '***

                        Result = HidD_GetAttributes(HIDHandle, MyHID.DeviceAttributes)

                        If Result Then
                            'Find out if the device matches the one we're looking for.
                            If (MyHID.DeviceAttributes.VendorID = MyVendorID) And _
                                (MyHID.DeviceAttributes.ProductID = MyProductID) Then

                                MyDeviceDetected = True

                                'Save the DevicePathName so OnDeviceChange() knows which name is my device.
                                MyDevicePathName = DevicePathName(MemberIndex)
                            Else

                                'It's not a match, so close the handle.
                                MyDeviceDetected = False

                                Result = CloseHandle(HIDHandle)
                            End If
                        Else
                            'There was a problem in retrieving the information. 
                            MyDeviceDetected = False
                            Result = CloseHandle(HIDHandle)
                        End If

                    End If


                    'Keep looking until we find the device or there are no more left to examine.
                    MemberIndex = MemberIndex + 1

                Loop Until ((MyDeviceDetected = True) Or _
                    (MemberIndex = UBound(DevicePathName) + 1))
            End If

            If MyDeviceDetected Then

                'The device was detected.
                'Register to receive notifications if the device is removed or attached.
                Success = MyDeviceManagement.RegisterForDeviceNotifications _
                    (MyDevicePathName, _
                    frmMain.Handle, _
                    HidGuid, _
                    DeviceNotificationHandle)

                'Learn the capabilities of the device.
                MyHID.Capabilities = MyHID.GetDeviceCapabilities(HIDHandle)

                If Success Then

                    'Find out if the device is a system mouse or keyboard.
                    HIDUsage = MyHID.GetHIDUsage(MyHID.Capabilities)

                    'Get and display the Input report buffer size.
                    GetInputReportBufferSize()

                    'Get handles to use in requesting Input and Output reports.

                    ReadHandle = CreateFile _
                        (MyDevicePathName, _
                        GENERIC_READ, _
                        FILE_SHARE_READ Or FILE_SHARE_WRITE, _
                        Security, _
                        OPEN_EXISTING, _
                        FILE_FLAG_OVERLAPPED, _
                        0)

                    If (ReadHandle = INVALID_HANDLE_VALUE) Then
                        ExclusiveAccess = True
                    Else

                        WriteHandle = CreateFile _
                             (MyDevicePathName, _
                             GENERIC_WRITE, _
                             FILE_SHARE_READ Or FILE_SHARE_WRITE, _
                             Security, _
                             OPEN_EXISTING, _
                             0, _
                             0)

                        '(optional)
                        'Flush any waiting reports in the input buffer.
                        MyHID.FlushQueue(ReadHandle)

                    End If

                End If

            Else
                If MostraMensaje Then
                    MsgBox("No se detecta la controladora en el USB...", MsgBoxStyle.Critical, "Deteccion HID")
                End If
            End If

            Return MyDeviceDetected

        Catch ex As Exception
            Call frmMain.HandleException("DetectarHID", ex)
        End Try
    End Function

    Private Sub GetInputReportBufferSize()

        'Purpose    : Finds and displays the number of Input buffers
        '           : (the number of Input reports the host will store). 

        Dim NumberOfInputBuffers As Integer

        Try
            'Get the number of input buffers.
            MyHID.GetNumberOfInputBuffers _
                (HIDHandle, _
                NumberOfInputBuffers)

        Catch ex As Exception
            Call frmMain.HandleException("GetInputReportBufferSize", ex)
        End Try

    End Sub


    Public Sub Close_USB()

        'Purpose    : Perform actions that must execute when the program ends.

        Try
            'Close open handles to the device.

            If (HIDHandle <> 0) Then
                CloseHandle(HIDHandle)
            End If

            If (ReadHandle <> 0) Then
                CloseHandle(ReadHandle)
            End If

            If (WriteHandle <> 0) Then
                CloseHandle(WriteHandle)
            End If

            'Stop receiving notifications.
            Call MyDeviceManagement.StopReceivingDeviceNotifications(DeviceNotificationHandle)

        Catch ex As Exception
            Call frmMain.HandleException("Close_USB", ex)
        End Try

    End Sub

    Private Function Escribir_USB() As Boolean
        Dim Count As Integer
        Dim OutputReportBuffer() As Byte
        Dim Success As Boolean

        Try
            Success = False

            'Don't attempt to exchange reports if valid handles aren't available
            '(as for a mouse or keyboard under Windows 2000/XP.)

            If ((ReadHandle <> INVALID_HANDLE_VALUE) And (WriteHandle <> INVALID_HANDLE_VALUE)) Then

                'Don't attempt to send an Output report if the HID has no Output report.
                If (MyHID.Capabilities.OutputReportByteLength > 0) Then

                    'Set the size of the Output report buffer. 
                    'Subtract 1 from the value in the Capabilities structure because 
                    'the array begins at index 0.

                    ReDim OutputReportBuffer(MyHID.Capabilities.OutputReportByteLength - 1)

                    'Store the report ID in the first byte of the buffer:
                    OutputReportBuffer(0) = 0

                    For Count = 1 To MyHID.Capabilities.OutputReportByteLength - 1
                        OutputReportBuffer(Count) = SendData(Count - 1)
                    Next Count

                    Dim myOutputReport As New Hid.OutputReport
                    Success = myOutputReport.Write(OutputReportBuffer, WriteHandle)

                    Escribir_USB = Success
                End If
            End If


        Catch ex As Exception
            Call frmMain.HandleException("Escribir_USB", ex)
        End Try
    End Function


    Private Function Leer_USB(ByRef ByteValue As Byte) As Boolean
        Dim Count As Integer
        Dim InputReportBuffer() As Byte
        Dim Success As Boolean

        Try
            'Read an Input report.
            Success = False
            'Don't attempt to send an Input report if the HID has no Input report.
            '(The HID spec requires all HIDs to have an interrupt IN endpoint,
            'which suggests that all HIDs must support Input reports.)

            If (MyHID.Capabilities.InputReportByteLength > 0) Then

                'Set the size of the Input report buffer. 
                'Subtract 1 from the value in the Capabilities structure because 
                'the array begins at index 0.

                ReDim InputReportBuffer(MyHID.Capabilities.InputReportByteLength - 1)


                'Read a report using a control transfer.
                Dim myInputReport As New Hid.InputReportViaControlTransfer

                'Read the report.
                myInputReport.Read(ReadHandle, HIDHandle, WriteHandle, MyDeviceDetected, InputReportBuffer, Success)

                Leer_USB = Success

                If Success Then
                    Debug.WriteLine(" Input Report ID: " & InputReportBuffer(0))
                    Debug.WriteLine(" Input Report Data:")

                    'txtBytesReceived.Text = ""
                    For Count = 1 To UBound(InputReportBuffer)
                        'Add a leading zero to values from 0 to F.
                        ByteValue = InputReportBuffer(Count)
                    Next Count
                End If
            End If

        Catch ex As Exception
            Call frmMain.HandleException("Leer_USB", ex)
        End Try

    End Function



    Public Function Leer_Posicion(ByVal nservo As Integer) As Byte
        'Leemos la posicion de los servos
        Try
            If MyDeviceDetected Then
                SendData(0) = Asc("P")              ' Cabecera ("P")
                SendData(1) = nservo                ' N de Servo al que accedemos

                If Escribir_USB() = False Then Exit Function
                If Leer_USB(Leer_Posicion) = False Then Exit Function
            End If

        Catch ex As Exception
            'Call frmMain.HandleException(frmMain.Name, ex)
        End Try

    End Function

    Public Sub Posicion_Servo(ByVal nServo As Integer, ByVal Posicion As Byte)
        Try
            SendData(0) = Asc("S")          ' Cabecera ("S")
            SendData(1) = nServo            ' N de Servo al que accedemos
            SendData(2) = Posicion          ' Enviamos la posicion
            ' Si no podemos escribir en el USB intentamos abrirlo
            If Escribir_USB() = False Then
                Call Close_USB()
                Call AbrirUSB(True)
            End If
        Catch ex As Exception
            Call frmMain.HandleException("Posicion_Servo", ex)
        End Try

    End Sub


    Public Sub Posicion_All_Servos()
        Try
            If MyDeviceDetected Then
                Dim TmpC As Integer

                SendData(0) = Asc("T")          ' Cabecera ("T")

                ' Preparacion del buffer
                For TmpC = 1 To 28
                    SendData(TmpC) = Servo(TmpC - 1)
                Next TmpC
                ' Si no podemos escribir en el USB intentamos abrirlo
                If Escribir_USB() = False Then
                    Call Close_USB()
                    Call AbrirUSB(True)
                End If


            End If

        Catch ex As Exception
            Call frmMain.HandleException("Posicion_All_Servos", ex)
        End Try

    End Sub


    Public Sub Comando_Inicio(ByVal nServo As Integer, ByVal Posicion As Byte)
        Try
            SendData(0) = Asc("I")          ' Cabecera ("I")
            SendData(1) = nServo            ' N de Servo al que accedemos
            SendData(2) = Posicion          ' Enviamos la posicion
            ' Si no podemos escribir en el USB intentamos abrirlo
            If Escribir_USB() = False Then
                Call Close_USB()
                Call AbrirUSB(True)
            End If
        Catch ex As Exception
            Call frmMain.HandleException("Comando_Inicio", ex)
        End Try

    End Sub


    Public Function Velocidad_Port(ByVal Velocidad As Byte) As Byte
        'Escribimos a velocidad del puerto serie

        ' Enviamos el comando "A" para asignar la velocidad de comunicacion

        ' 0 - 9.600
        ' 1 - 19.200
        ' 2 - 38.400
        ' 3 - 57.600
        ' 4 - 115.200
        Try
            SendData(0) = Asc("A")          ' Cabecera ("A")
            SendData(1) = Velocidad         ' Velocidad Serie
            ' Si no podemos escribir en el USB intentamos abrirlo
            If Escribir_USB() = False Then
                Call Close_USB()
                Call AbrirUSB(False)
            End If
        Catch ex As Exception
            Call frmMain.HandleException("Velocidad_port", ex)
        End Try

    End Function
End Module
